@@ -1,13 +1,13 @@
 /*
  * IMAPFolder.java
- * Copyright(C) 2003, 2004 Chris Burdess <dog@gnu.org>
+ * Copyright (C) 2003, 2004 Chris Burdess <dog@gnu.org>
  * 
  * This file is part of GNU JavaMail, a library.
  * 
  * GNU JavaMail is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
- *(at your option) any later version.
+ * (at your option) any later version.
  * 
  * GNU JavaMail is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -16,7 +16,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  * 
  * As a special exception, if you link this library with other files to
  * produce an executable, this library does not by itself cause the
@@ -33,7 +33,9 @@ import java.text.DateFormat;
 import java.text.SimpleDateFormat;
 import java.util.ArrayList;
 import java.util.Date;
+import java.util.Iterator;
 import java.util.List;
+import java.util.Map;
 import javax.mail.Address;
 import javax.mail.FetchProfile;
 import javax.mail.Flags;
@@ -85,11 +87,10 @@ import gnu.inet.imap.Quota;
  * The folder class implementing the IMAP4rev1 mail protocol.
  *
  * @author <a href='mailto:dog@gnu.org'>Chris Burdess</a>
- * @version 0.1
  */
 public class IMAPFolder 
-extends Folder
-implements UIDFolder
+  extends Folder
+  implements UIDFolder
 {
 
   /**
@@ -98,7 +99,7 @@ implements UIDFolder
   protected String path;
 
   /**
-   * The type of this folder(HOLDS_MESSAGES or HOLDS_FOLDERS).
+   * The type of this folder (HOLDS_MESSAGES or HOLDS_FOLDERS).
    */
   protected int type;
 
@@ -265,17 +266,13 @@ implements UIDFolder
               {
                ((IMAPStore) store).processAlerts();
               }
-            type = 0;
+            type = Folder.HOLDS_FOLDERS;
             if (entries.length > 0)
               {
-                if (entries[0].isNoinferiors())
+                if (!"".equals(path) && entries[0].isNoinferiors())
                   {
                     type |= Folder.HOLDS_MESSAGES;
                   }
-                if (entries[0].isNoselect())
-                  {
-                    type |= Folder.HOLDS_FOLDERS;
-                  }
               }
             else
               {
@@ -862,27 +859,29 @@ implements UIDFolder
           }
         list.add(buffer.toString());
       }
-    addTerm(term, list);
+    boolean isIMAPSearch = addTerm(term, list);
     String[] criteria = new String[list.size()];
     list.toArray(criteria);
     IMAPStore s = (IMAPStore) store;
     IMAPConnection connection = s.getConnection();
     try
       {
-        int[] mn = null;
-        synchronized (connection)
+        Message[] messages;
+        if (isIMAPSearch && criteria.length > 0)
           {
-            mn = connection.search(null, criteria);
-          }
-        Message[] messages = new Message[mn.length];
-        for (int i = 0; i < mn.length; i++)
-          {
-            messages[i] = new IMAPMessage(this, mn[i]);
-          }
-        if (connection.alertsPending())
-          {
-            s.processAlerts();
+            int[] mn = null;
+            synchronized (connection)
+              {
+                mn = connection.search(null, criteria);
+              }
+            messages = new Message[mn.length];
+            for (int i = 0; i < mn.length; i++)
+              messages[i] = new IMAPMessage(this, mn[i]);
+            if (connection.alertsPending())
+              s.processAlerts();
           }
+        else
+          messages = (msgs != null) ? msgs : getMessages();
         // Enforce final constraints
         return super.search(term, messages);
       }
@@ -897,15 +896,17 @@ implements UIDFolder
    * Note that this is not sufficient to enforce all the constraints imposed
    * by the SearchTerm structures - this is why we finally call
    * <code>super.search()</code> in the search method.
+   * @return true if all the terms can be represented in IMAP
    */
-  private void addTerm(SearchTerm term, List list)
+  private boolean addTerm(SearchTerm term, List list)
   {
     if (term instanceof AndTerm)
       {
         SearchTerm[] terms = ((AndTerm) term).getTerms();
         for (int i = 0; i < terms.length; i++)
           {
-            addTerm(terms[i], list);
+            if (!addTerm(terms[i], list))
+              return false;
           }
       }
     else if (term instanceof OrTerm)
@@ -914,13 +915,15 @@ implements UIDFolder
         SearchTerm[] terms = ((OrTerm) term).getTerms();
         for (int i = 0; i < terms.length; i++)
           {
-            addTerm(terms[i], list);
+            if (!addTerm(terms[i], list))
+              return false;
           }
       }
     else if (term instanceof NotTerm)
       {
         list.add(IMAPConstants.SEARCH_NOT);
-        addTerm(((NotTerm) term).getTerm(), list);
+        if (!addTerm(((NotTerm) term).getTerm(), list))
+          return false;
       }
     else if (term instanceof FlagTerm)
       {
@@ -981,34 +984,22 @@ implements UIDFolder
         Address address = ((AddressTerm) term).getAddress();
         StringBuffer criterion = new StringBuffer();
         if (term instanceof FromTerm)
-          {
-            criterion.append(IMAPConstants.SEARCH_FROM);
-          }
+          criterion.append(IMAPConstants.SEARCH_FROM);
         else if (term instanceof RecipientTerm)
           {
             Message.RecipientType rt =
-             ((RecipientTerm) term).getRecipientType();
+              ((RecipientTerm) term).getRecipientType();
             if (rt == Message.RecipientType.TO)
-              {
-                criterion.append(IMAPConstants.SEARCH_TO);
-              }
+              criterion.append(IMAPConstants.SEARCH_TO);
             else if (rt == Message.RecipientType.CC)
-              {
-                criterion.append(IMAPConstants.SEARCH_CC);
-              }
+              criterion.append(IMAPConstants.SEARCH_CC);
             else if (rt == Message.RecipientType.BCC)
-              {
-                criterion.append(IMAPConstants.SEARCH_BCC);
-              }
+              criterion.append(IMAPConstants.SEARCH_BCC);
             else
-              {
-                criterion = null;
-              }
+              criterion = null;
           }
         else
-          {
-            criterion = null;
-          }
+          criterion = null;
         if (criterion != null)
           {
             criterion.append(' ');
@@ -1017,6 +1008,8 @@ implements UIDFolder
             criterion.append('"');
             list.add(criterion.toString());
           }
+        else
+          return false;
       }
     else if (term instanceof ComparisonTerm)
       {
@@ -1035,9 +1028,7 @@ implements UIDFolder
                 criterion.append(' ');
               }
             if (term instanceof SentDateTerm)
-              {
-                criterion.append("SENT");
-              }
+              criterion.append("SENT");
             switch (comparison)
               {
               case ComparisonTerm.EQ:
@@ -1102,6 +1093,8 @@ implements UIDFolder
                   }
                 list.add(criterion.toString());
               }
+            else
+              return false;
           }
       }
     else if (term instanceof StringTerm)
@@ -1129,9 +1122,7 @@ implements UIDFolder
             criterion.append("Message-ID");
           }
         else
-          {
-            criterion = null; // TODO StringAddressTerms?
-          }
+          criterion = null; // TODO StringAddressTerms?
         if (criterion != null)
           {
             criterion.append(' ');
@@ -1140,7 +1131,12 @@ implements UIDFolder
             criterion.append('"');
             list.add(criterion.toString());
           }
+        else
+          return false;
       }
+    else
+      return false;
+    return true;
   }
 
   public boolean isSubscribed()
@@ -1179,6 +1175,9 @@ implements UIDFolder
   public Folder[] list(String pattern) 
     throws MessagingException 
   {
+    char sep = getSeparator();
+    String spec = ("".equals(path)) ? path : 
+      new StringBuffer(path).append(sep).append(pattern).toString();
     IMAPStore s = (IMAPStore) store;
     IMAPConnection connection = s.getConnection();
     try
@@ -1186,7 +1185,7 @@ implements UIDFolder
         ListEntry[] entries;
         synchronized (connection)
           {
-            entries = connection.list(path, pattern);
+            entries = connection.list("", spec);
           }
         if (connection.alertsPending())
           {
@@ -1206,6 +1205,9 @@ implements UIDFolder
   public Folder[] listSubscribed(String pattern) 
     throws MessagingException 
   {
+    char sep = getSeparator();
+    String spec = ("".equals(path)) ? path :
+      new StringBuffer(path).append(sep).append(pattern).toString();
     IMAPStore s = (IMAPStore) store;
     IMAPConnection connection = s.getConnection();
     try
@@ -1213,7 +1215,7 @@ implements UIDFolder
         ListEntry[] entries = null;
         synchronized (connection)
           {
-            entries = connection.lsub(path, pattern);
+            entries = connection.lsub("", spec);
           }
         if (connection.alertsPending())
           {
@@ -1575,5 +1577,208 @@ implements UIDFolder
     
     return ids.length;
   }
+
+  /**
+   * Returns the access control list for this folder.
+   * This is an array of access control entries.
+   * @deprecated this API will probably change incompatibly soon
+   */
+  public ACL[] getACL()
+    throws MessagingException
+  {
+    if (!isOpen())
+      {
+        return null;
+      }
+    
+    IMAPStore s = (IMAPStore) store;
+    IMAPConnection connection = s.getConnection();
+    List acc = new ArrayList();
+    try
+      {
+        Map acl = connection.getacl(path);
+        for (Iterator i = acl.entrySet().iterator(); i.hasNext(); )
+          {
+            Map.Entry entry = (Map.Entry) i.next();
+            String name = (String) entry.getKey();
+            Integer rights = (Integer) entry.getValue();
+            ACL ace = new ACL(name);
+            if (rights != null)
+              {
+                ace.rights = new Rights();
+                ace.rights.rights = rights.intValue();
+              }
+            acc.add(ace);
+          }
+      }
+    catch (IOException e)
+      {
+        throw new MessagingException(e.getMessage(), e);
+      }
+    ACL[] ret = new ACL[acc.size()];
+    acc.toArray(ret);
+    return ret;
+  }
+
+  /**
+   * Adds the specified access control entry to this folder.
+   * @param ace the access control entry
+   * @deprecated this API will probably change incompatibly soon
+   */
+  public void addACL(ACL ace)
+    throws MessagingException
+  {
+    if (!isOpen())
+      {
+        return;
+      }
+    
+    IMAPStore s = (IMAPStore) store;
+    IMAPConnection connection = s.getConnection();
+    try
+      {
+        Rights aceRights = ace.getRights();
+        if (aceRights != null)
+          {
+            int rights = connection.listrights(path, ace.name);
+            rights |= aceRights.rights;
+            if (!connection.setacl(path, ace.name, rights))
+              {
+                throw new MessagingException("can't set ACL");
+              }
+          }
+      }
+    catch (IOException e)
+      {
+        throw new MessagingException(e.getMessage(), e);
+      }
+  }
+
+  /**
+   * Removes the ACL for the given principal.
+   * @param name the name of the principal
+   * @deprecated this API will probably change incompatibly soon
+   */
+  public void removeACL(String name)
+    throws MessagingException
+  {
+    if (!isOpen())
+      {
+        return;
+      }
+    
+    IMAPStore s = (IMAPStore) store;
+    IMAPConnection connection = s.getConnection();
+    try
+      {
+        if (!connection.deleteacl(path, name))
+          {
+            throw new MessagingException("can't delete ACL");
+          }
+      }
+    catch (IOException e)
+      {
+        throw new MessagingException(e.getMessage(), e);
+      }
+  }
+
+  /**
+   * Adds the rights for the specified access control entry.
+   * @param ace the access control entry
+   * @deprecated this API will probably change incompatibly soon
+   */
+  public void addRights(ACL ace)
+    throws MessagingException
+  {
+    addACL(ace);
+  }
+
+  /**
+   * Removes the rights specified in the given access control entry from the
+   * principal.
+   * @param ace the access control entry
+   * @deprecated this API will probably change incompatibly soon
+   */
+  public void removeRights(ACL ace)
+    throws MessagingException
+  {
+    if (!isOpen())
+      {
+        return;
+      }
+    
+    IMAPStore s = (IMAPStore) store;
+    IMAPConnection connection = s.getConnection();
+    try
+      {
+        Rights aceRights = ace.getRights();
+        if (aceRights != null)
+          {
+            int rights = connection.listrights(path, ace.name);
+            rights -= aceRights.rights;
+            if (!connection.setacl(path, ace.name, rights))
+              {
+                throw new MessagingException("can't set ACL");
+              }
+          }
+      }
+    catch (IOException e)
+      {
+        throw new MessagingException(e.getMessage(), e);
+      }
+  }
+
+  /**
+   * Returns the rights currently assigned to the given principal.
+   * @param name the name of the principal
+   * @deprecated this API will probably change incompatibly soon
+   */
+  public Rights listRights(String name)
+    throws MessagingException
+  {
+    if (!isOpen())
+      {
+        return null;
+      }
+    
+    IMAPStore s = (IMAPStore) store;
+    IMAPConnection connection = s.getConnection();
+    try
+      {
+        Rights rights = new Rights();
+        rights.rights = connection.listrights(path, name);
+        return rights;
+      }
+    catch (IOException e)
+      {
+        throw new MessagingException(e.getMessage(), e);
+      }
+  }
+  
+  /**
+   * Returns the rights assigned to the currently authenticated principal.
+   * @deprecated this API will probably change incompatibly soon
+   */
+  public Rights myRights()
+    throws MessagingException
+  {
+    if (!isOpen())
+      {
+        return null;
+      }
+    
+    IMAPStore s = (IMAPStore) store;
+    IMAPConnection connection = s.getConnection();
+    try
+      {
+        Rights rights = new Rights();
+        rights.rights = connection.myrights(path);
+        return rights;
+      }
+    catch (IOException e)
+      {
+        throw new MessagingException(e.getMessage(), e);
+      }
+  }
   
 }
