@@ -1,6 +1,6 @@
 /*
  * Provider.java
- * Copyright (C) 2002 The Free Software Foundation
+ * Copyright (C) 2002, 2005 The Free Software Foundation
  * 
  * This file is part of GNU JavaMail, a library.
  * 
@@ -16,7 +16,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  * As a special exception, if you link this library with other files to
  * produce an executable, this library does not by itself cause the
@@ -32,7 +32,7 @@ package javax.mail;
  * messages.
  *
  * @author <a href="mailto:dog@gnu.org">Chris Burdess</a>
- * @version 1.3
+ * @version 1.4
  */
 public class Provider
 {
@@ -61,8 +61,17 @@ public class Provider
   private String vendor;
   private String version;
 
-  Provider(Type type, String protocol, String className, String vendor,
-           String version)
+  /**
+   * Creates a new provider of the given type and protocol.
+   * @param type the provider type
+   * @param protocol the protocol URL scheme
+   * @param className the name of the implementing class
+   * @param vendor the implementation vendor
+   * @param version the implementation version
+   * @since JavaMail 1.4
+   */
+  public Provider(Type type, String protocol, String className, String vendor,
+                  String version)
   {
     this.type = type;
     this.protocol = protocol;
